# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging
import os

import fixtures

_TRUE_VALUES = ('True', 'true', '1', 'yes')
_FALSE_VALUES = ('False', 'false', '0', 'no')
_BASE_LOG_LEVELS = ('DEBUG', 'INFO', 'WARN', 'WARNING', 'ERROR', 'CRITICAL')
_LOG_LEVELS = dict((n, getattr(logging, n)) for n in _BASE_LOG_LEVELS)
_LOG_LEVELS.update({
    'TRACE': 5,
})


def _try_int(value):
    """Try to make some value into an int."""
    try:
        return int(value)
    except (ValueError, TypeError):
        return None


class ConfigureLogging(fixtures.Fixture):
    """Configure logging.

    The behavior is managed through two environment variables. If
    ``OS_DEBUG`` is true then the logging level is set to debug. If
    ``OS_LOG_CAPTURE`` is true a FakeLogger is configured. Alternatively,
    ``OS_DEBUG`` can be set to an explicit log level, such as ``INFO``.

    "True" values include ``True``, ``true``, ``1`` and ``yes``.
    Valid log levels include ``DEBUG``, ``INFO``, ``WARNING``, ``ERROR``,
    ``TRACE`` and ``CRITICAL`` (or any other valid integer logging level).

    .. py:attribute:: logger

       The logger fixture, if it is created.

    .. py:attribute:: level

       ``logging.DEBUG`` if debug logging is enabled, otherwise
       the log level specified by ``OS_DEBUG``, otherwise ``None``.

    :param format: The logging format string to use.

    """

    DEFAULT_FORMAT = "%(levelname)8s [%(name)s] %(message)s"
    """Default log format"""

    def __init__(self, format=DEFAULT_FORMAT):
        super(ConfigureLogging, self).__init__()
        self._format = format
        self.level = None
        _os_debug = os.environ.get('OS_DEBUG')
        _os_level = _try_int(_os_debug)
        if _os_debug in _TRUE_VALUES:
            self.level = logging.DEBUG
        elif _os_level is not None:
            self.level = _os_level
        elif _os_debug in _LOG_LEVELS:
            self.level = _LOG_LEVELS[_os_debug]
        elif _os_debug and _os_debug not in _FALSE_VALUES:
            raise ValueError('OS_DEBUG=%s is invalid.' % (_os_debug))
        self.capture_logs = os.environ.get('OS_LOG_CAPTURE') in _TRUE_VALUES
        self.logger = None

    def setUp(self):
        super(ConfigureLogging, self).setUp()
        if self.capture_logs:
            self.logger = self.useFixture(
                fixtures.FakeLogger(
                    format=self._format,
                    level=self.level,
                    nuke_handlers=True,
                )
            )
        else:
            logging.basicConfig(format=self._format, level=self.level)



# -*- coding: utf-8 -*-

# Copyright 2014 Deutsche Telekom AG
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
import os
import unittest
from unittest import mock

import fixtures
import testtools

from oslotest import base


class TestBaseTestCase(testtools.TestCase):

    class FakeTestCase(base.BaseTestCase):
        def test_fake_test(self):
            pass

    @mock.patch('os.environ.get')
    @mock.patch('oslotest.timeout.Timeout.useFixture')
    @mock.patch('fixtures.Timeout')
    def test_timeout(self, fixture_timeout_mock, fixture_mock, env_get_mock):
        env_get_mock.return_value = 1
        tc = self.FakeTestCase("test_fake_test")
        tc._set_timeout()
        env_get_mock.assert_called_once_with('OS_TEST_TIMEOUT', 0)
        fixture_timeout_mock.assert_called_once_with(1, gentle=True)
        self.assertEqual(1, fixture_mock.call_count)

    @mock.patch('os.environ.get')
    def test_fake_logs_default(self, env_get_mock):
        # without debug and log capture
        env_get_mock.side_effect = lambda value, default=None: {
            'OS_DEBUG': 0, 'OS_LOG_CAPTURE': 0}.get(value, default)
        tc = self.FakeTestCase("test_fake_test")
        tc.setUp()
        env_get_mock.assert_any_call('OS_LOG_CAPTURE')
        env_get_mock.assert_any_call('OS_DEBUG')
        self.assertFalse(tc.log_fixture.capture_logs)
        self.assertIsNone(tc.log_fixture.logger)

    @mock.patch('os.environ.get')
    @mock.patch('logging.basicConfig')
    def test_fake_logs_with_debug(self, basic_logger_mock, env_get_mock):
        env_get_mock.side_effect = lambda value, default=None: {
            'OS_DEBUG': 'True', 'OS_LOG_CAPTURE': 0}.get(value, default)
        tc = self.FakeTestCase("test_fake_test")
        tc.setUp()
        env_get_mock.assert_any_call('OS_LOG_CAPTURE')
        env_get_mock.assert_any_call('OS_DEBUG')
        basic_logger_mock.assert_called_once_with(format=base._LOG_FORMAT,
                                                  level=logging.DEBUG)

    @mock.patch('os.environ.get')
    @mock.patch.object(FakeTestCase, 'useFixture')
    def test_fake_logs_with_log_cap(self, fixture_mock, env_get_mock):
        env_get_mock.side_effect = lambda value: {'OS_DEBUG': 0,
                                                  'OS_LOG_CAPTURE': 'True'
                                                  }.get(value)
        tc = self.FakeTestCase("test_fake_test")
        tc.setUp()
        env_get_mock.assert_any_call('OS_LOG_CAPTURE')
        env_get_mock.assert_any_call('OS_DEBUG')
        self.assertEqual(5, fixture_mock.call_count)

    def test_mock_patch_cleanup_on_teardown(self):
        # create an object and save its reference
        class Sub(object):
            pass

        obj = Sub()
        obj.value = obj.backup = object()

        # patch the object
        mock.patch.object(obj, 'value').start()
        self.assertNotEqual(obj.backup, obj.value)

        # run a test case
        loader = unittest.defaultTestLoader
        suite = loader.loadTestsFromTestCase(self.FakeTestCase)
        suite.run(unittest.TestResult())

        # check that mock patches are cleaned up
        self.assertEqual(obj.backup, obj.value)

    @mock.patch('os.environ')
    def test_capture_output_disabled(self, mock_env):
        mock_env.get.return_value = ''
        tc = self.FakeTestCase("test_fake_test")
        tc.setUp()
        self.assertIsNone(tc.output_fixture.stdout)
        self.assertIsNone(tc.output_fixture.stderr)

    @mock.patch('os.environ')
    def test_enabled(self, mock_env):
        mock_env.get.return_value = 'True'
        tc = self.FakeTestCase("test_fake_test")
        tc.setUp()
        self.assertIsNotNone(tc.output_fixture.stdout)
        self.assertIsNotNone(tc.output_fixture.stderr)


class TestManualMock(base.BaseTestCase):

    def setUp(self):
        # Create a cleanup to undo a patch() call *before* calling the
        # base class version of setup().
        patcher = mock.patch('os.environ.keys')
        patcher.start()
        self.addCleanup(patcher.stop)
        super(TestManualMock, self).setUp()
        self.useFixture(fixtures.MockPatch('fixtures.Timeout'))

    def test_mock_patch_manually(self):
        # Verify that if a test instance creates its own mock and
        # calls start/stop itself we don't get an error.
        patcher = mock.patch('os.environ.get')
        patcher.start()
        self.addCleanup(patcher.stop)


class TestTempFiles(base.BaseTestCase):
    def test_create_unicode_files(self):
        files = [["no_approve", 'ಠ_ಠ']]
        temps = self.create_tempfiles(files)
        self.assertEqual(1, len(temps))
        with open(temps[0], 'rb') as f:
            contents = f.read()
        self.assertEqual('ಠ_ಠ', str(contents, encoding='utf-8'))

    def test_create_unicode_files_encoding(self):
        files = [["embarrassed", '⊙﹏⊙', 'utf-8']]
        temps = self.create_tempfiles(files)
        self.assertEqual(1, len(temps))
        with open(temps[0], 'rb') as f:
            contents = f.read()
        self.assertEqual('⊙﹏⊙', str(contents, encoding='utf-8'))

    def test_create_unicode_files_multi_encoding(self):
        files = [
            ["embarrassed", '⊙﹏⊙', 'utf-8'],
            ['abc', 'abc', 'ascii'],
        ]
        temps = self.create_tempfiles(files)
        self.assertEqual(2, len(temps))
        for i, (basename, raw_contents, raw_encoding) in enumerate(files):
            with open(temps[i], 'rb') as f:
                contents = f.read()
            if not isinstance(raw_contents, str):
                raw_contents = str(raw_contents, encoding=raw_encoding)
            self.assertEqual(str(contents, encoding=raw_encoding),
                             raw_contents)

    def test_create_bad_encoding(self):
        files = [["hrm", 'ಠ~ಠ', 'ascii']]
        self.assertRaises(UnicodeError, self.create_tempfiles, files)

    def test_prefix(self):
        files = [["testing", '']]
        temps = self.create_tempfiles(files)
        self.assertEqual(1, len(temps))
        basename = os.path.basename(temps[0])
        self.assertTrue(basename.startswith('testing'))

    def test_wrong_length(self):
        files = [["testing"]]
        self.assertRaises(ValueError, self.create_tempfiles, files)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""Utilities functions for working with oslo.config from the tool scripts.
"""

import os

from oslo_config import cfg

DEFAULT_CONFIG_FILES = [
    './oslo.conf',
    os.path.expanduser('~/.oslo.conf'),
]


def get_config_parser():
    conf = cfg.ConfigOpts()
    conf.register_cli_opt(
        cfg.StrOpt(
            'repo_root',
            default='.',
            help='directory containing the git repositories',
        )
    )
    return conf


def parse_arguments(conf):
    # Look for a few configuration files, and load the ones we find.
    default_config_files = [
        f
        for f in DEFAULT_CONFIG_FILES
        if os.path.exists(f)
    ]
    return conf(
        project='oslo',
        default_config_files=default_config_files,
    )



# Copyright 2017 Cloudbase Solutions Srl
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools
from unittest import mock

from oslotest import mock_fixture

# NOTE(claudiub): this needs to be called before any mock.patch calls are
# being done, and especially before the test class loads. This fixes
# the mock.patch autospec issue:
# https://github.com/testing-cabal/mock/issues/396
# TODO(claudiub): Remove this once the fix has been merged and released.
mock_fixture.patch_mock_module()


class Foo(object):
    def bar(self, a, b, c, d=None):
        pass

    @classmethod
    def classic_bar(cls, a, b, c, d=None):
        pass

    @staticmethod
    def static_bar(a, b, c, d=None):
        pass


class MockSanityTestCase(testtools.TestCase):

    def setUp(self):
        super(MockSanityTestCase, self).setUp()
        self.useFixture(mock_fixture.MockAutospecFixture())

    def _check_autospeced_foo(self, foo):
        for method_name in ['bar', 'classic_bar', 'static_bar']:
            mock_method = getattr(foo, method_name)

            # check that the methods are callable with correct args.
            mock_method(mock.sentinel.a, mock.sentinel.b, mock.sentinel.c)
            mock_method(mock.sentinel.a, mock.sentinel.b, mock.sentinel.c,
                        d=mock.sentinel.d)
            mock_method.assert_has_calls([
                mock.call(mock.sentinel.a, mock.sentinel.b, mock.sentinel.c),
                mock.call(mock.sentinel.a, mock.sentinel.b, mock.sentinel.c,
                          d=mock.sentinel.d)])

            # assert that TypeError is raised if the method signature is not
            # respected.
            self.assertRaises(TypeError, mock_method)
            self.assertRaises(TypeError, mock_method, mock.sentinel.a)
            self.assertRaises(TypeError, mock_method, a=mock.sentinel.a)
            self.assertRaises(TypeError, mock_method, mock.sentinel.a,
                              mock.sentinel.b, mock.sentinel.c,
                              e=mock.sentinel.e)

        # assert that AttributeError is raised if the method does not exist.
        self.assertRaises(AttributeError, getattr, foo, 'lish')

    def _check_mock_autospec_all_members(self, mock_cls):
        for spec in [Foo, Foo()]:
            foo = mock_cls(autospec=spec)
            self._check_autospeced_foo(foo)
            self._check_autospeced_foo(foo())

    def test_mock_autospec_all_members(self):
        self._check_mock_autospec_all_members(mock.Mock)

    def test_magic_mock_autospec_all_members(self):
        self._check_mock_autospec_all_members(mock.MagicMock)

    @mock.patch.object(Foo, 'static_bar')
    @mock.patch.object(Foo, 'classic_bar')
    @mock.patch.object(Foo, 'bar')
    def test_patch_autospec_class(self, mock_meth, mock_cmeth, mock_smeth):
        foo = Foo()
        self._check_autospeced_foo(foo)

    def test_patch_autospec_multiple(self):
        with mock.patch.multiple(Foo, bar=mock.DEFAULT,
                                 classic_bar=mock.DEFAULT,
                                 static_bar=mock.DEFAULT):
            foo = Foo()
            self._check_autospeced_foo(foo)

    @mock.patch.object(Foo, 'static_bar', autospec=False)
    @mock.patch.object(Foo, 'classic_bar', autospec=False)
    @mock.patch.object(Foo, 'bar', autospec=False)
    def test_patch_autospec_class_false(self, mock_meth, mock_cmeth,
                                        mock_smeth):
        foo = Foo()
        # we're checking that method signature is not enforced.
        foo.bar()
        mock_meth.assert_called_once_with()
        foo.classic_bar()
        mock_cmeth.assert_called_once_with()
        foo.static_bar()
        mock_smeth.assert_called_once_with()

    @mock.patch.object(Foo, 'lish', create=True)
    def test_patch_create(self, mock_lish):
        foo = Foo()

        foo.lish()
        mock_lish.assert_called_once_with()



# -*- coding: utf-8 -*-

#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sys
from unittest import mock

import testtools

from oslotest import output


class CaptureOutputTest(testtools.TestCase):

    @mock.patch('os.environ')
    def test_disabled_env(self, mock_env):
        mock_env.get.return_value = ''
        f = output.CaptureOutput()
        f.setUp()
        self.assertIsNone(f.stdout)
        self.assertIsNone(f.stderr)
        self.assertIsNot(sys.stdout, f.stdout)
        self.assertIsNot(sys.stderr, f.stderr)

    @mock.patch('os.environ')
    def test_enabled_env(self, mock_env):
        mock_env.get.return_value = 'True'
        f = output.CaptureOutput()
        f.setUp()
        self.assertIsNotNone(f.stdout)
        self.assertIsNotNone(f.stderr)
        self.assertIs(sys.stdout, f.stdout)
        self.assertIs(sys.stderr, f.stderr)

    @mock.patch('os.environ')
    def test_disabled_explicit(self, mock_env):
        mock_env.get.side_effect = AssertionError('should not be called')
        f = output.CaptureOutput(do_stdout=False, do_stderr=False)
        f.setUp()
        self.assertIsNone(f.stdout)
        self.assertIsNone(f.stderr)
        self.assertIsNot(sys.stdout, f.stdout)
        self.assertIsNot(sys.stderr, f.stderr)

    @mock.patch('os.environ')
    def test_ensabled_explicit(self, mock_env):
        mock_env.get.side_effect = AssertionError('should not be called')
        f = output.CaptureOutput(do_stdout=True, do_stderr=True)
        f.setUp()
        self.assertIsNotNone(f.stdout)
        self.assertIsNotNone(f.stderr)
        self.assertIs(sys.stdout, f.stdout)
        self.assertIs(sys.stderr, f.stderr)



# Copyright (c) 2013 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Common utilities used in testing"""

import logging
from unittest import mock

import fixtures
from oslotest import createfile
from oslotest import log
from oslotest import output
from oslotest import timeout

import testtools

LOG = logging.getLogger(__name__)

_TRUE_VALUES = ('True', 'true', '1', 'yes')
_LOG_FORMAT = "%(levelname)8s [%(name)s] %(message)s"


class BaseTestCase(testtools.TestCase):
    """Base class for unit test classes.

    If the environment variable ``OS_TEST_TIMEOUT`` is set to an
    integer value (**seconds**), a timer is configured to control how long
    individual test cases can run. This lets tests fail for taking too
    long, and prevents deadlocks from completely hanging test runs.

    The class variable ``DEFAULT_TIMEOUT`` can be set to configure
    a test suite default test value for cases in which ``OS_TEST_TIMEOUT``
    is not set. It defaults to ``0``, which means no timeout.

    The class variable ``TIMEOUT_SCALING_FACTOR`` can be set on an
    individual test class for tests that reasonably take longer than
    the rest of the test suite so that the overall timeout can be
    kept small. It defaults to ``1``.

    If the environment variable ``OS_STDOUT_CAPTURE`` is set, a fake
    stream replaces ``sys.stdout`` so the test can look at the output
    it produces.

    If the environment variable ``OS_STDERR_CAPTURE`` is set, a fake
    stream replaces ``sys.stderr`` so the test can look at the output
    it produces.

    If the environment variable ``OS_DEBUG`` is set to a true value,
    debug logging is enabled. Alternatively, the ``OS_DEBUG``
    environment variable can be set to a valid log level.

    If the environment variable ``OS_LOG_CAPTURE`` is set to a true
    value, a logging fixture is installed to capture the log output.

    Uses the fixtures_ module to configure a :class:`NestedTempFile`
    to ensure that all temporary files are created in an isolated
    location.

    Uses the fixtures_ module to configure a :class:`TempHomeDir` to
    change the ``HOME`` environment variable to point to a temporary
    location.

    PLEASE NOTE:
    Usage of this class may change the log level globally by setting the
    environment variable ``OS_DEBUG``. A mock of ``time.time`` will be called
    many more times than might be expected because it's called often by the
    logging module. A usage of such a mock should be avoided when a test needs
    to verify logging behavior or counts the number of invocations. A
    workaround is to overload the ``_fake_logs`` function in a base class but
    this will deactivate fake logging globally.

    .. _fixtures: https://pypi.org/project/fixtures

    """
    DEFAULT_TIMEOUT = 0
    TIMEOUT_SCALING_FACTOR = 1

    def __init__(self, *args, **kwds):
        super(BaseTestCase, self).__init__(*args, **kwds)

        # This is the number of characters shown when two objects do not
        # match for assertDictEqual, assertMultiLineEqual, and
        # assertSequenceEqual. The default is 640 which is too
        # low for comparing most dicts
        self.maxDiff = 10000

    def addCleanup(self, function, *args, **kwargs):
        # NOTE(dims): This is a hack for Mitaka. We'll need to undo this as
        # early as possible in Newton and advertise that this hack will not
        # be supported anymore.
        if (hasattr(self, '_cleanups') and
                isinstance(self._cleanups, list)):
            if not self._cleanups:
                # Ensure that the mock.patch.stopall cleanup is registered
                # before any addCleanup() methods have a chance to register
                # other things to be cleaned up, so it is called last. This
                # allows tests to register their own cleanups with a mock.stop
                # method so those mocks are not included in the stopall set.
                super(BaseTestCase, self).addCleanup(mock.patch.stopall)
        else:
            LOG.error('Unable to patch test case. '
                      'mock.patch.stopall cleanup was not registered.')
        super(BaseTestCase, self).addCleanup(function, *args, **kwargs)

    def setUp(self):
        super(BaseTestCase, self).setUp()
        self._set_timeout()
        self._fake_output()
        self._fake_logs()
        self.useFixture(fixtures.NestedTempfile())
        self.useFixture(fixtures.TempHomeDir())

    def _set_timeout(self):
        self.useFixture(timeout.Timeout(
            default_timeout=self.DEFAULT_TIMEOUT,
            scaling_factor=self.TIMEOUT_SCALING_FACTOR,
        ))

    def _fake_output(self):
        self.output_fixture = self.useFixture(output.CaptureOutput())

    def _fake_logs(self):
        self.log_fixture = self.useFixture(log.ConfigureLogging())

    def create_tempfiles(self, files, ext='.conf', default_encoding='utf-8'):
        """Safely create temporary files.

        :param files: Sequence of tuples containing (filename, file_contents).
        :type files: list of tuple
        :param ext: File name extension for the temporary file.
        :type ext: str
        :param default_encoding: Default file content encoding when it is
                                 not provided, used to decode the tempfile
                                 contents from a text string into a binary
                                 string.
        :type default_encoding: str
        :return: A list of str with the names of the files created.
        """
        tempfiles = []
        for f in files:
            if len(f) == 3:
                basename, contents, encoding = f
            else:
                basename, contents = f
                encoding = default_encoding
            fix = self.useFixture(createfile.CreateFileWithContent(
                filename=basename,
                contents=contents,
                ext=ext,
                encoding=encoding,
            ))
            tempfiles.append(fix.path)
        return tempfiles



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import sys

import fixtures


class DisableModuleFixture(fixtures.Fixture):
    """A fixture to provide support for unloading/disabling modules."""

    def __init__(self, module, *args, **kwargs):
        super(DisableModuleFixture, self).__init__(*args, **kwargs)
        self.module = module

    def setUp(self):
        """Ensure ImportError for the specified module."""
        super(DisableModuleFixture, self).setUp()

        cleared_modules = {}

        for name in list(sys.modules.keys()):
            if name == self.module or name.startswith(self.module + '.'):
                cleared_modules[name] = sys.modules.pop(name)

        finder = _NoModuleFinder(self.module)
        sys.meta_path.insert(0, finder)

        self.addCleanup(sys.meta_path.remove, finder)
        self.addCleanup(sys.modules.update, cleared_modules)


class _NoModuleFinder(object):
    """Disallow further imports of 'module'."""

    def __init__(self, module):
        self.module = module

    def find_spec(self, fullname, path, target):
        if fullname == self.module or fullname.startswith(self.module + '.'):
            raise ImportError




# Copyright 2017 Cloudbase Solutions Srl
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import functools
from unittest import mock

import fixtures


def _lazy_autospec_method(mocked_method, original_method, eat_self):
    if mocked_method._mock_check_sig.__dict__.get('autospeced'):
        return

    _lazy_autospec = mock.create_autospec(original_method)
    if eat_self:
        # consume self argument.
        _lazy_autospec = functools.partial(_lazy_autospec, None)

    def _autospeced(*args, **kwargs):
        _lazy_autospec(*args, **kwargs)

    # _mock_check_sig is called by the mock's __call__ method.
    # which means that if a method is not called, _autospeced is not
    # called.
    _autospeced.__dict__['autospeced'] = True
    mocked_method._mock_check_sig = _autospeced


class _AutospecMockMixin(object):
    """Mock object that lazily autospecs the given spec's methods."""

    def __init__(self, *args, **kwargs):
        super(_AutospecMockMixin, self).__init__(*args, **kwargs)
        autospec = kwargs.get('autospec')
        self.__dict__['_autospec'] = autospec
        _mock_methods = self.__dict__['_mock_methods']
        if _mock_methods:
            # this will allow us to be able to set _mock_check_sig if
            # the spec_set argument has been given.
            _mock_methods.append('_mock_check_sig')

        # callable mocks with autospecs (e.g.: the given autospec is a class)
        # should have their return values autospeced as well.
        if autospec:
            self.return_value.__dict__['_autospec'] = autospec

    def __getattr__(self, name):
        attr = super(_AutospecMockMixin, self).__getattr__(name)

        original_spec = self.__dict__['_autospec']
        if not original_spec:
            return attr

        if not hasattr(original_spec, name):
            raise AttributeError(name)

        # check if the original attribute is callable, and the mock was not
        # autospeced already.
        original_attr = getattr(original_spec, name)
        if callable(original_attr):
            # lazily autospec callable attribute.
            eat_self = mock._must_skip(
                original_spec, name,
                isinstance(original_spec, type)
            )

            _lazy_autospec_method(attr, original_attr, eat_self)

        return attr


class _AutospecMock(_AutospecMockMixin, mock.Mock):
    pass


class _AutospecMagicMock(_AutospecMockMixin, mock.MagicMock):
    pass


class MockAutospecFixture(fixtures.Fixture):
    """A fixture to add / fix the autospec feature into the mock library.

    The current version of the mock library has a few unaddressed issues, which
    can lead to erroneous unit tests, and can hide actual issues. This fixture
    is to be removed once these issues have been addressed in the mock library.

    Issue addressed by the fixture:

    * mocked method's signature checking:
        - https://github.com/testing-cabal/mock/issues/393
        - mock can only accept a spec object / class, and it makes sure that
          that attribute exists, but it does not check whether the given
          attribute is callable, or if its signature is respected in any way.
        - adds autospec argument. If the autospec argument is given, the
          mocked method's signature is also checked.
    """

    def setUp(self):
        super(MockAutospecFixture, self).setUp()

        # patch both external and internal usage of Mock / MagicMock.
        self.useFixture(
            fixtures.MonkeyPatch(
                'unittest.mock.Mock',
                _AutospecMock))
        self.useFixture(
            fixtures.MonkeyPatch(
                'unittest.mock.MagicMock',
                _AutospecMagicMock))


class _patch(mock._patch):
    """Patch class with working autospec functionality.

    Currently, mock.patch functionality doesn't handle the autospec parameter
    properly (the self argument is not consumed, causing assertions to fail).
    Until the issue is addressed in the mock library, this should be used
    instead.
    https://github.com/testing-cabal/mock/issues/396
    """

    def __enter__(self):
        # NOTE(claudiub): we're doing the autospec checks here so unit tests
        # have a chance to set up mocks in advance (e.g.: mocking platform
        # specific libraries, which would cause the patch to fail otherwise).

        # By default, autospec is None. We will consider it as True.
        autospec = True if self.autospec is None else self.autospec

        # in some cases, autospec cannot be set to True.
        skip_autospec = (getattr(self, attr) for attr in
                         ['new_callable', 'create', 'spec'])
        # NOTE(claudiub): The "new" argument is always mock.DEFAULT, unless
        # explicitly set otherwise.
        if self.new is not mock.DEFAULT or any(skip_autospec):
            # cannot autospec if new, new_callable, or create arguments given.
            autospec = False
        elif self.attribute:
            target = getattr(self.getter(), self.attribute, None)
            if isinstance(target, mock.Mock):
                # NOTE(claudiub): shouldn't autospec already mocked targets.
                # this can cause some issues. There are quite a few tests
                # which patch mocked methods.
                autospec = False

        # NOTE(claudiub): reset the self.autospec property, so we can handle
        # the autospec scenario ourselves.
        self.autospec = None

        if autospec:
            target = self.getter()
            original_attr = getattr(target, self.attribute)
            eat_self = mock._must_skip(
                target,
                self.attribute,
                isinstance(target, type)
            )

            new = super(_patch, self).__enter__()

            # NOTE(claudiub): mock.patch.multiple will cause new to be a
            # dict.
            mocked_method = (new[self.attribute] if isinstance(new, dict)
                             else new)
            _lazy_autospec_method(mocked_method, original_attr, eat_self)
            return new
        else:
            return super(_patch, self).__enter__()


def _safe_attribute_error_wrapper(func):
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except AttributeError:
            pass

    return wrapper


def patch_mock_module():
    """Replaces the mock.patch class."""
    mock._patch = _patch

    # NOTE(claudiub): mock cannot autospec partial functions properly,
    # especially those created by LazyLoader objects (scheduler client),
    # as it will try to copy the partial function's __name__ (which they do
    # not have).
    mock._copy_func_details = _safe_attribute_error_wrapper(
        mock._copy_func_details)



# -*- coding: utf-8 -*-

#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

import testtools

from oslotest import timeout


class TimeoutTestCase(testtools.TestCase):

    @mock.patch('os.environ.get')
    @mock.patch.object(timeout.Timeout, 'useFixture')
    @mock.patch('fixtures.Timeout')
    def test_timeout(self, fixture_timeout_mock, fixture_mock, env_get_mock):
        env_get_mock.return_value = 1
        tc = timeout.Timeout()
        tc.setUp()
        env_get_mock.assert_called_once_with('OS_TEST_TIMEOUT', 0)
        fixture_timeout_mock.assert_called_once_with(1, gentle=True)
        self.assertEqual(1, fixture_mock.call_count)

    @mock.patch('os.environ.get')
    @mock.patch.object(timeout.Timeout, 'useFixture')
    @mock.patch('fixtures.Timeout')
    def test_no_timeout(self, fixture_timeout_mock, fixture_mock,
                        env_get_mock):
        # Returning 0 means we don't install the timeout
        env_get_mock.return_value = 0
        tc = timeout.Timeout()
        tc.setUp()
        env_get_mock.assert_called_once_with('OS_TEST_TIMEOUT', 0)
        self.assertEqual(0, fixture_timeout_mock.call_count)
        self.assertEqual(0, fixture_mock.call_count)

    @mock.patch('os.environ.get')
    @mock.patch.object(timeout.Timeout, 'useFixture')
    @mock.patch('fixtures.Timeout')
    def test_timeout_default(
            self, fixture_timeout_mock, fixture_mock, env_get_mock):
        env_get_mock.return_value = 5
        tc = timeout.Timeout(default_timeout=5)
        tc.setUp()
        env_get_mock.assert_called_once_with('OS_TEST_TIMEOUT', 5)
        fixture_timeout_mock.assert_called_once_with(5, gentle=True)
        self.assertEqual(1, fixture_mock.call_count)

    @mock.patch('os.environ.get')
    @mock.patch.object(timeout.Timeout, 'useFixture')
    @mock.patch('fixtures.Timeout')
    def test_timeout_bad_default(
            self, fixture_timeout_mock, fixture_mock, env_get_mock):
        env_get_mock.return_value = 'invalid'
        tc = timeout.Timeout(default_timeout='invalid')
        tc.setUp()
        env_get_mock.assert_called_once_with('OS_TEST_TIMEOUT', 0)
        self.assertEqual(0, fixture_timeout_mock.call_count)
        self.assertEqual(0, fixture_mock.call_count)

    @mock.patch('os.environ.get')
    @mock.patch.object(timeout.Timeout, 'useFixture')
    @mock.patch('fixtures.Timeout')
    def test_timeout_scaling(
            self, fixture_timeout_mock, fixture_mock, env_get_mock):
        env_get_mock.return_value = 2
        tc = timeout.Timeout(scaling_factor=1.5)
        tc.setUp()
        env_get_mock.assert_called_once_with('OS_TEST_TIMEOUT', 0)
        fixture_timeout_mock.assert_called_once_with(3, gentle=True)
        self.assertEqual(1, fixture_mock.call_count)

    @mock.patch('os.environ.get')
    @mock.patch.object(timeout.Timeout, 'useFixture')
    @mock.patch('fixtures.Timeout')
    def test_timeout_bad_scaling(
            self, fixture_timeout_mock, fixture_mock, env_get_mock):
        env_get_mock.return_value = 2
        tc = timeout.Timeout(scaling_factor='invalid')
        tc.setUp()
        env_get_mock.assert_called_once_with('OS_TEST_TIMEOUT', 0)
        fixture_timeout_mock.assert_called_once_with(2, gentle=True)
        self.assertEqual(1, fixture_mock.call_count)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import fixtures

import os


class Timeout(fixtures.Fixture):
    """Set the maximum length of time for the test to run.

    Uses OS_TEST_TIMEOUT to get the timeout.

    """

    def __init__(self, default_timeout=0, scaling_factor=1):
        super(Timeout, self).__init__()
        try:
            self._default_timeout = int(default_timeout)
        except ValueError:
            # If timeout value is invalid do not set a timeout.
            self._default_timeout = 0
        self._scaling_factor = scaling_factor

    def setUp(self):
        super(Timeout, self).setUp()
        test_timeout = os.environ.get('OS_TEST_TIMEOUT', self._default_timeout)
        try:
            test_timeout = int(test_timeout)
        except ValueError:
            # If timeout value is invalid use the default timeout.
            test_timeout = self._default_timeout
        try:
            scaled_timeout = int(test_timeout * self._scaling_factor)
        except ValueError:
            # If scaling factor is invalid, use the basic test timeout.
            scaled_timeout = test_timeout
        if scaled_timeout > 0:
            self.useFixture(fixtures.Timeout(scaled_timeout, gentle=True))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os

import fixtures

_TRUE_VALUES = ('True', 'true', '1', 'yes')


class CaptureOutput(fixtures.Fixture):
    """Optionally capture the output streams.

    The behavior is managed through arguments to the constructor. The
    default behavior is controlled via two environment variables. If
    ``OS_STDOUT_CAPTURE`` is true then stdout is captured and if
    ``OS_STDERR_CAPTURE`` is true then stderr is captured.

    "True" values include ``True``, ``true``, ``1``, and ``yes``.

    :param do_stdout: Whether to capture stdout.
    :type do_stdout: bool
    :param do_stderr: Whether to capture stderr.
    :type do_stderr: bool

    .. py:attribute:: stdout

       The ``stream`` attribute from a :class:`StringStream` instance
       replacing stdout.

    .. py:attribute:: stderr

       The ``stream`` attribute from a :class:`StringStream` instance
       replacing stderr.

    """

    def __init__(self, do_stdout=None, do_stderr=None):
        super(CaptureOutput, self).__init__()
        if do_stdout is None:
            self.do_stdout = (os.environ.get('OS_STDOUT_CAPTURE')
                              in _TRUE_VALUES)
        else:
            self.do_stdout = do_stdout
        if do_stderr is None:
            self.do_stderr = (os.environ.get('OS_STDERR_CAPTURE')
                              in _TRUE_VALUES)
        else:
            self.do_stderr = do_stderr
        self.stdout = None
        self.stderr = None

    def setUp(self):
        super(CaptureOutput, self).setUp()
        if self.do_stdout:
            self._stdout_fixture = fixtures.StringStream('stdout')
            self.stdout = self.useFixture(self._stdout_fixture).stream
            self.useFixture(fixtures.MonkeyPatch('sys.stdout', self.stdout))
        if self.do_stderr:
            self._stderr_fixture = fixtures.StringStream('stderr')
            self.stderr = self.useFixture(self._stderr_fixture).stream
            self.useFixture(fixtures.MonkeyPatch('sys.stderr', self.stderr))



# -*- coding: utf-8 -*-

# Copyright 2014 Deutsche Telekom AG
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

from oslotest import base
from oslotest import createfile


class CreateFileWithContentTest(base.BaseTestCase):

    def test_create_unicode_files(self):
        f = createfile.CreateFileWithContent(
            "no_approve",
            'ಠ_ಠ',
        )
        f.setUp()
        with open(f.path, 'rb') as f:
            contents = f.read()
        self.assertEqual('ಠ_ಠ', str(contents, encoding='utf-8'))

    def test_create_unicode_files_encoding(self):
        f = createfile.CreateFileWithContent(
            "embarrassed", '⊙﹏⊙', encoding='utf-8',
        )
        f.setUp()
        with open(f.path, 'rb') as f:
            contents = f.read()
        self.assertEqual('⊙﹏⊙', str(contents, encoding='utf-8'))

    def test_create_bad_encoding(self):
        f = createfile.CreateFileWithContent(
            "hrm", 'ಠ~ಠ', encoding='ascii',
        )
        self.assertRaises(UnicodeError, f.setUp)

    def test_prefix(self):
        f = createfile.CreateFileWithContent('testing', '')
        f.setUp()
        basename = os.path.basename(f.path)
        self.assertTrue(basename.startswith('testing'))

    def test_ext(self):
        f = createfile.CreateFileWithContent('testing', '', ext='.ending')
        f.setUp()
        basename = os.path.basename(f.path)
        self.assertTrue(basename.endswith('.ending'))



# -*- coding: utf-8 -*-

# Copyright 2014 Deutsche Telekom AG
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
from unittest import mock

import testtools

from oslotest import log


class ConfigureLoggingTestCase(testtools.TestCase):

    @mock.patch('os.environ.get')
    def test_fake_logs_default(self, env_get_mock):
        # without debug and log capture
        env_get_mock.side_effect = lambda value, default=None: {
            'OS_DEBUG': 0, 'OS_LOG_CAPTURE': 0}.get(value, default)
        f = log.ConfigureLogging()
        f.setUp()
        env_get_mock.assert_any_call('OS_LOG_CAPTURE')
        env_get_mock.assert_any_call('OS_DEBUG')
        self.assertFalse(f.capture_logs)
        self.assertIsNone(f.logger)

    @mock.patch('os.environ.get')
    @mock.patch('logging.basicConfig')
    def test_fake_logs_with_debug(self, basic_logger_mock, env_get_mock):
        env_get_mock.side_effect = lambda value, default=None: {
            'OS_DEBUG': 'True', 'OS_LOG_CAPTURE': 0}.get(value, default)
        f = log.ConfigureLogging()
        f.setUp()
        env_get_mock.assert_any_call('OS_LOG_CAPTURE')
        env_get_mock.assert_any_call('OS_DEBUG')
        basic_logger_mock.assert_called_once_with(
            format=log.ConfigureLogging.DEFAULT_FORMAT,
            level=logging.DEBUG)

    @mock.patch('os.environ.get')
    @mock.patch('logging.basicConfig')
    def test_fake_logs_with_warning(self, basic_logger_mock, env_get_mock):
        env_get_mock.side_effect = lambda value, default=None: {
            'OS_DEBUG': 'WARNING', 'OS_LOG_CAPTURE': 0}.get(value, default)
        f = log.ConfigureLogging()
        f.setUp()
        env_get_mock.assert_any_call('OS_LOG_CAPTURE')
        env_get_mock.assert_any_call('OS_DEBUG')
        basic_logger_mock.assert_called_once_with(
            format=log.ConfigureLogging.DEFAULT_FORMAT,
            level=logging.WARNING)

    @mock.patch('os.environ.get')
    @mock.patch('logging.basicConfig')
    def test_fake_logs_with_trace_int(self, basic_logger_mock, env_get_mock):
        env_get_mock.side_effect = lambda value, default=None: {
            'OS_DEBUG': '5', 'OS_LOG_CAPTURE': 0}.get(value, default)
        f = log.ConfigureLogging()
        f.setUp()
        env_get_mock.assert_any_call('OS_LOG_CAPTURE')
        env_get_mock.assert_any_call('OS_DEBUG')
        basic_logger_mock.assert_called_once_with(
            format=log.ConfigureLogging.DEFAULT_FORMAT,
            level=5)

    @mock.patch('os.environ.get')
    @mock.patch('logging.basicConfig')
    def test_fake_logs_with_debug_int(self, basic_logger_mock, env_get_mock):
        env_get_mock.side_effect = lambda value, default=None: {
            'OS_DEBUG': '10', 'OS_LOG_CAPTURE': 0}.get(value, default)
        f = log.ConfigureLogging()
        f.setUp()
        env_get_mock.assert_any_call('OS_LOG_CAPTURE')
        env_get_mock.assert_any_call('OS_DEBUG')
        basic_logger_mock.assert_called_once_with(
            format=log.ConfigureLogging.DEFAULT_FORMAT,
            level=logging.DEBUG)

    @mock.patch('os.environ.get')
    def test_fake_logs_with_log_capture(self, env_get_mock):
        env_get_mock.side_effect = lambda value: {'OS_DEBUG': 0,
                                                  'OS_LOG_CAPTURE': 'True'
                                                  }[value]
        f = log.ConfigureLogging()
        f.setUp()
        env_get_mock.assert_any_call('OS_LOG_CAPTURE')
        env_get_mock.assert_any_call('OS_DEBUG')
        self.assertIsNotNone(f.logger)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os
import tempfile

import fixtures


class CreateFileWithContent(fixtures.Fixture):
    """Create a temporary file with the given content.

    Creates a file using a predictable name, to be used by tests for
    code that need a filename to load data or otherwise interact with
    the real filesystem.

    .. warning::

       It is the responsibility of the caller to ensure that the file
       is removed.

    Users of this fixture may also want to use
    :class:`fixtures.NestedTempfile` to set the temporary directory
    somewhere safe and to ensure the files are cleaned up.

    .. py:attribute:: path

       The canonical name of the file created.

    :param filename: Base file name or full literal path to the file
        to be created.
    :param contents: The data to write to the file. Unicode data will
        be encoded before being written.
    :param ext: An extension to add to filename.
    :param encoding: An encoding to use for unicode data (ignored for
        byte strings).

    """

    def __init__(self, filename, contents, ext='.conf', encoding='utf-8'):
        self._filename = filename
        self._contents = contents
        self._ext = ext
        self._encoding = encoding

    def setUp(self):
        super(CreateFileWithContent, self).setUp()
        contents = self._contents
        if isinstance(contents, str):
            contents = contents.encode(self._encoding)
        if not os.path.isabs(self._filename):
            (fd, self.path) = tempfile.mkstemp(prefix=self._filename,
                                               suffix=self._ext)
        else:
            self.path = self._filename + self._ext
            fd = os.open(self.path, os.O_CREAT | os.O_WRONLY)
        try:
            os.write(fd, contents)
        finally:
            os.close(fd)
